"""Counter module

Defines the Traverser base class and UserCounter

$Id: Counter.py,v 1.1.1.1 2001/02/06 18:01:34 seant Exp $
"""

__version__='$Revision: 1.1.1.1 $'[11:-2]

from Globals import HTMLFile, MessageDialog, Persistent
from Acquisition import Implicit
from OFS.SimpleItem import Item
from string import split, strip
from ZPublisher import BeforeTraverse
import os, time

class Traverser(Persistent, Item):
    """Class for overriding container's __before_traverse__

    Containers are expected to have at most one instance of any particular
    subclass, with Id equal to the meta_type of the subclass."""

    meta_type='Traverser'
    priority = 100

    __ac_permissions__=()

    def addToContainer(self, container):
        container._setObject(self.id, self)
        self.manage_afterAdd(self, container)

    def manage_addToContainer(self, container, nextURL=''):
        if nextURL:
            if hasattr(getattr(container, 'aq_base', container), self.id):
                return MessageDialog(title='Item Exists',
                  message='This object already contains a %s' % self.meta_type,
                  action=nextURL)
        self.addToContainer(container)
        if nextURL:    
            return MessageDialog(title='Item Added',
              message='This object now has a %s' % self.meta_type, 
              action=nextURL)

    def manage_beforeDelete(self, item, container):
        if item is self:
            self._unregister(container)

    def manage_afterAdd(self, item, container):
        if item is self:
            self._register(container)

    def _register(self, container):
        id = self.id
        if callable(id): id = id()

        # We want the original object, not stuff in between
        container = container.this()
        hook = BeforeTraverse.NameCaller(id)
        BeforeTraverse.registerBeforeTraverse(container, hook,
                                              self._getKey(container),
                                              self.priority)

    def _unregister(self, container):
        BeforeTraverse.unregisterBeforeTraverse(container, 
                                              self._getKey(container))

    def _getKey(self, container=None):
        id = self.id
        cid = container and container.id
        if callable(id): id = id()
        if cid and callable(cid): cid = cid()
        return (cid or '') + id
        
    def _setId(self, id):
        self.id = id

class UserCounter(Traverser, Implicit):
    """Counter.UserCounter object

    When a traversal passes through the container of this UserCounter
    this object records the last access of time of the IP making the
    request that initiates the traversal"""

    meta_type = 'User Counter'
    title = 'Counts the number of active users on a site'
    expire_time = 600.0 # seconds
    _v_ips = {}

    manage_options=({'label':'Statistics', 'action':'manage_main'},
                    {'label':'Edit',       'action':'manage_editForm'},)

    manage = manage_main = HTMLFile('www/UserCounterStats', globals())
    manage_editForm = HTMLFile('www/UserCounterEdit', globals())

    def __init__(self, id, title, expire_time):
        '''Title'''
        self.id = id
        self.title = strip(title)
        self.expire_time = expire_time

    def manage_edit(self, title, expire_time=600.0, REQUEST=None):
        '''Set the title, base, and path'''
        self.__init__(self.id, title, expire_time)
        if REQUEST:
          return MessageDialog(title='UserCounter changed.',
            message='The title is now "%s"<br>'
                    'The expire time is now "%s"<br>' % (title, expire_time),
            action='%s/manage_editForm' % REQUEST['URL1'])

    def manage_resetCounter(self, REQUEST=None):
        '''Clear out the cache'''
        self._v_ips = {}
        if REQUEST:
          return MessageDialog(title='Counter Reset',
            message='Number of active users is now: %d<br>'
                    '<small>Which is probably just you.</small><br>' % \
                        self.user_count(),
            action='%s/manage_main' % REQUEST['URL1'])

    def __setstate__(self, state):
        '''Make sure we set our volatle instance map rather than use 
           the class variable which would apply to all instances
        '''
        self._v_ips = {}
        Persistent.__setstate__(self, state)

    def __call__(self, client, request, response=None):
        '''Traversing'''
        # Refresh the client's last access time
        remote_ip = request['REMOTE_ADDR']
        self._v_ips[remote_ip] = time.time()

    def user_count(self):
        '''Return the number of active users'''
        self.expire()
        return len(self._v_ips.keys())

    def expire(self):
        '''Expire the requests that are older than N'''
        threshold = time.time() - self.expire_time
        for ip, t in self._v_ips.items():
            if t < threshold:
                del self._v_ips[ip]

    def active_users(self):
        '''Return the number of active users'''
        return self._v_ips.items()

    def get_size(self):
        '''Make FTP happy'''
        return 0

def manage_addUserCounter(self, id, title='', expire_after=600.0, REQUEST=None, **ignored):
    """Create the counter"""
    sr=UserCounter(id, title, expire_after)
    if REQUEST:
        return sr.manage_addToContainer(self.this(),
                                        '%s/manage_main' % REQUEST['URL1'])
    else:
        sr.manage_addToContainer(self.this())

constructors = (
  ('manage_addUserCounterForm', HTMLFile('www/UserCounterAdd', globals())),
  ('manage_addUserCounter', manage_addUserCounter),
)
